@@ -2,6 +2,9 @@
 use strict;
 use warnings;
 
+my $qr = qr{unblessed};
+my $qr_blessed = qr{blessed}i;
+
 my %tests = (
 
     hash => [
@@ -74,8 +77,7 @@ EOM
 
     regexp => [
         <<'EOM',
-        my $string = 'unblessed';
-        qr{$string}
+        qr{unblessed}
 EOM
         <<"EOM",
 ---
@@ -86,8 +88,7 @@ EOM
 
     regexp_blessed => [
         <<'EOM',
-        my $string = 'blessed';
-        bless qr{$string}, "Foo"
+        bless qr{blessed}, "Foo"
 EOM
         <<"EOM",
 ---
@@ -119,7 +120,7 @@ EOM
             return $args{x} + $args{y};
         }
 EOM
-        qr{- &1 !{1,2}perl/code \|-.*return.*args.*x.*\+.*y}s,
+        qr{- &1 !perl/code \|-.*return.*args.*x.*\+.*y}s,
         { load_code => 1 },
     ],
 
@@ -130,7 +131,7 @@ EOM
             return $args{x} - $args{y};
         }, "I::Am::Code"
 EOM
-        qr{- &1 !{1,2}perl/code:I::Am::Code \|-.*return.*args.*x.*\-.*y}s,
+        qr{- &1 !perl/code:I::Am::Code \|-.*return.*args.*x.*\-.*y}s,
         { load_code => 1 },
     ],
 
