@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package YAML::PP::Perl;
 
-our $VERSION = 'v0.38.1'; # VERSION
+our $VERSION = '0.012'; # VERSION
 
 use base 'Exporter';
 use base 'YAML::PP';
@@ -63,9 +63,7 @@ YAML::PP::Perl - Convenience module for loading and dumping Perl objects
 =head1 DESCRIPTION
 
 This is just for convenience. It will create a YAML::PP object using the
-default schema (C<Core>) and the L<YAML::PP::Schema::Perl> schema.
-
-See L<YAML::PP::Schema::Perl> for documentation.
+default schema (currently C<Core>) and the L<YAML::PP::Schema::Perl> schema.
 
 =head1 METHODS
 
